////////////////////////////////////////////////////////////////////////
//Beispiel Kapitel Fraktale: HenonFluchtgeschwindigkeit
///////////////////////////////////////////////////////////////////////
// Größe des Ausschnitts, das geschieht
// über wh. Dieser Wert wird beim Vergrößern oder Verkleinern entsprechend
// verändert. (in MousePressed())
double xmin = -6; 
double ymin = -6; 
// Groesse des bereiches
double wh = 12;

// parameter des Henon-Systems
double alpha = 0.2;
double beta = 1.01;

// maximale Anzahl Iterationen, die für die komplexe Zahl ausgeführt wird
// danach wird die Farbe anhand des Betrags der Zahl gewählt. Die Anzahl 
// kann verändert werden, entsprechend mehr Details sieht man dann in 
// manchen Teilen der Menge
int maxiterations = 50;

// ist der Betrag eines Punktes (x*x+y*y)groesser als dieser Wert, dann 
// geht er gegen Unendlich und wir brechen ab
double MassFuerUnendlich = 100;

//---------------------------------------------------------------------

int henonIteration(double x, double y) {

  // nun wird für einen Punkt der Ebene getestet, wohin er bei der 
  // Iteration strebt. Je nachdem wird er dann später eingefärbt
  
  int n = 0;
  while (n < maxiterations) {
    double xNeu = 1 - alpha*x*x + y;
    double yNeu = beta*x;
    if (xNeu*xNeu + yNeu*yNeu > MassFuerUnendlich) {
      break;  // Bail
    }
    x = xNeu;
    y = yNeu;
    n++;
  }
  return n;
}

//------------------------------------------------

color colorize(int n) {
  // jedes Pixel wird entsprechend der Anzahl Iterationen eingefärbt
  // bis entweder der Wert des Betrages Null oder über der
  // Schwelle "MassFuerUnendlich" ist
  if (n == maxiterations) return color(0, 0, 0);
  return color(255*n/maxiterations,255,255);
}

//------------------------------------------------

void setup() {
  size(600, 600);
  noLoop();
  colorMode(HSB);
}

//---------------------------------------------------
void draw() {

  loadPixels();
  // x wird von xmin nach ixmax iteriert
  double xmax = xmin + wh;
  double ymax = ymin + wh;

  // die inkrmentelle Erhoehung für x und y
  double dx = (xmax - xmin) / (width);
  double dy = (ymax - ymin) / (height);

  double y = ymin;
  for (int j = 0; j < height; j++) {
    double x = xmin;
    for (int i = 0; i < width; i++) {
      int n = henonIteration(x, y);
      pixels[i+j*width] = colorize(n);
      if ((x<=0) && (x+dx>0) || ((y<=0) && (y+dy>0)))
        pixels[i+j*width] = color(0,0,maxiterations);
      x += dx;
    }
    y += dy;
  }
  updatePixels();
}

//------------------------------------------------

void mousePressed() {

  // berechne, wo der Mauszeiger im gezeigten Ausschnitt der komplexen
  // Zahlenebene ist (Transformation Pixelposition nach (x,y))
  double x = xmin + wh*mouseX/width;
  double y = ymin + wh*mouseY/height;

  // linke Maustaste: verkleinere den gezeigten Ausschnitt um 0.66
  // rechte Taste: vergrößere den gezeigten Ausschnitt entsprechend
  if (mouseButton == LEFT) wh*=0.66; 
  else wh/=0.66;

  // bestimme den neuen gezeigten Ausschnitt
  xmin = x-wh/2.0;
  ymin = y-wh/2.0;

  redraw();
}  

//-------------------------------------------------

void keyPressed() {
  if (key == '+') maxiterations *= 1.2;
  if (key == '-') maxiterations /= 1.2;
  if (key == 's') save("henon.png");
  redraw();
}

//-------------------------------------------------
